from typing import List

from .core.models import Section
from .utils.files import get_file
from .utils.daemon import Daemon


class Systemd:
    """
    Create systemd service file
    """

    def __init__(self, name: str, sections: List[Section]):
        """
        Create systemd service file

        Args:
            name (str, required): service name
            sections (List[Section], required): list of sections
        """

        self._name = name
        self._sections = sections

        self._service = None

        self.daemon = Daemon(name)

    def create(self) -> str:
        """
        Build systemd service file

        Returns:
            str: systemd unit file
        """

        header = f'# {self._name}.service\n\n'
        sections = "\n\n".join([str(s) for s in self._sections])
        footer = f"\n\n# Via systemd_py"

        self._service = header + sections + footer

        return self._service

    def save(self, path: str) -> None:
        """
        Save systemd service file

        Args:
            path (str, required): path to save file

        Returns:
            None
        """

        if self._service is None:
            self.create()

        path = get_file(self._name, path)

        with open(path, 'w') as f:
            f.write(self._service)

        print(f'Saved {path}')

    def save_in_os(self) -> None:
        """
        Save systemd service file in /etc/systemd/system

        Returns:
            None
        """

        self.save('/etc/systemd/system')

    def start_service(self):
        """
        Start service

        Returns:
            None
        """

        self.daemon.reload()
        self.daemon.enable()
        self.daemon.start()

    def __str__(self):
        if self._service is None:
            self.create()

        return self._service

    def __repr__(self):
        return self.__str__()



"""
SystemdPy
"""

from .main import Systemd

from .core.models import Install
from .core.models import Unit
from .core.models import Service
from .core.models import Socket

from .builders import InstallBuilder
from .builders import ServiceBuilder
from .builders import SocketBuilder
from .builders import UnitBuilder



from typing import Literal

from ._enums import _Enum


class Type(_Enum):
    """
    Service Type
    """

    simple: Literal['simple'] = 'simple'
    forking: Literal['forking'] = 'forking'
    oneshot: Literal['oneshot'] = 'oneshot'
    dbus: Literal['dbus'] = 'dbus'
    notify: Literal['notify'] = 'notify'
    idle: Literal['idle'] = 'idle'



"""
# Unit Builder 🏗️
Builder for the `uni` section of a systemd service file.
"""

from typing import Optional
from typing import List

from ._builder import _Builder
from ..core.models import Unit


class UnitBuilder(_Builder):
    """
    Builder for the `unit` section of a systemd service file.
    """

    __slots__ = (
        "_description",
        "_documentation",
        "_requires",
        "_wants",
        "_binds_to",
        "_before",
        "_after",
        "_conflicts",
        "_condition",
        "_assert_",
    )

    @property
    def allowed_none_fields(self) -> List[str]:
        """
        Returns a list of fields that are allowed to be None


        :return: List of fields that are allowed to be None
        :rtype: List[str]
        """

        return [
            "_documentation", "_requires", "_wants", "_binds_to", "_before",
            "_after", "_conflicts", "_condition", "_assert_",
        ]

    def build(self) -> Unit:
        """
        Builds the `unit` section of a systemd service file.

        :return: The `unit` section of a systemd service file
        :rtype: Unit
        """

        self._check()
        return Unit(
            description=self._description,
            documentation=self._documentation,
            requires=self._requires,
            wants=self._wants,
            binds_to=self._binds_to,
            before=self._before,
            after=self._after,
            conflicts=self._conflicts,
            condition=self._condition,
            assert_=self._assert_,
        )

    def __init__(self):
        self._description: str = ...
        self._documentation: Optional[str] = ...
        self._requires: Optional[List[str]] = ...
        self._wants: Optional[List[str]] = ...
        self._binds_to: Optional[List[str]] = ...
        self._before: Optional[List[str]] = ...
        self._after: Optional[List[str]] = ...
        self._conflicts: Optional[List[str]] = ...
        self._condition: Optional[str] = ...
        self._assert_: Optional[str] = ...

    def with_description(self, description: str) -> 'UnitBuilder':
        """
        Sets the `description` field of the `unit` section of a systemd service file.

        :param description: The `description` field of the `unit` section of a systemd service file
        :type description: str

        :return: The `unit` section of a systemd service file
        :rtype: Unit

        :return: The SocketBuilder object
        :rtype: SocketBuilder
        """

        self._description = description
        return self

    def with_documentation(self, documentation: str) -> 'UnitBuilder':
        """
        Sets the `documentation` field of the `unit` section of a systemd service file.

        :param documentation: The `documentation` field of the `unit` section of a systemd service file
        :type documentation: str

        :return: The `unit` section of a systemd service file
        :rtype: Unit
        """

        self._documentation = documentation
        return self

    def with_requires(self, requires: List[str]) -> 'UnitBuilder':
        """
        Sets the `requires` field of the `unit` section of a systemd service file.

        :param requires: The `requires` field of the `unit` section of a systemd service file
        :type requires: List[str]

        :return: The `unit` section of a systemd service file
        :rtype: Unit
        """

        self._requires = requires
        return self

    def with_wants(self, wants: List[str]) -> 'UnitBuilder':
        """
        Sets the `wants` field of the `unit` section of a systemd service file.

        :param wants: The `wants` field of the `unit` section of a systemd service file
        :type wants: List[str]

        :return: The `unit` section of a systemd service file
        :rtype: Unit
        """

        self._wants = wants
        return self

    def with_binds_to(self, binds_to: List[str]) -> 'UnitBuilder':
        """
        Sets the `binds_to` field of the `unit` section of a systemd service file.

        :param binds_to: The `binds_to` field of the `unit` section of a systemd service file
        :type binds_to: List[str]

        :return: The `unit` section of a systemd service file
        :rtype: Unit
        """

        self._binds_to = binds_to
        return self

    def with_before(self, before: List[str]) -> 'UnitBuilder':
        """
        Sets the `before` field of the `unit` section of a systemd service file.

        :param before: The `before` field of the `unit` section of a systemd service file
        :type before: List[str]

        :return: The `unit` section of a systemd service file
        :rtype: Unit
        """

        self._before = before
        return self

    def with_after(self, after: List[str]) -> 'UnitBuilder':
        """
        Sets the `after` field of the `unit` section of a systemd service file.

        :param after: The `after` field of the `unit` section of a systemd service file
        :type after: List[str]

        :return: The `unit` section of a systemd service file
        :rtype: Unit
        """

        self._after = after
        return self

    def with_conflicts(self, conflicts: List[str]) -> 'UnitBuilder':
        """
        Sets the `conflicts` field of the `unit` section of a systemd service file.

        :param conflicts: The `conflicts` field of the `unit` section of a systemd service file
        :type conflicts: List[str]

        :return: The `unit` section of a systemd service file
        :rtype: Unit
        """

        self._conflicts = conflicts
        return self

    def with_condition(self, condition: str) -> 'UnitBuilder':
        """
        Sets the `condition` field of the `unit` section of a systemd service file.

        :param condition: The `condition` field of the `unit` section of a systemd service file
        :type condition: str

        :return: The `unit` section of a systemd service file
        :rtype: Unit
        """

        self._condition = condition
        return self

    def with_assert(self, assert_: str) -> 'UnitBuilder':
        """
        Sets the `assert` field of the `unit` section of a systemd service file.

        :param assert_: The `assert` field of the `unit` section of a systemd service file
        :type assert_: str

        :return: The `unit` section of a systemd service file
        :rtype: Unit
        """

        self._assert_ = assert_
        return self



from enum import Enum
from typing import List, Union, Optional, Dict
from pydantic import BaseModel
from pydantic import Field

from ...exceptions import SystemdPyError


class Section(BaseModel):
    """
    Systemd Section
    """

    extra: Optional[Dict[str, str]] = Field(
        None,
        title="Extra",
        description="Extra directives"
    )

    def _to_str(self, value: Union[str, List, int, float, bool]) -> str:
        if isinstance(value, str):
            return value
        elif isinstance(value, list):
            return " ".join(value)
        elif isinstance(value, bool):
            return str(value).lower()
        elif isinstance(value, (int, float)):
            return str(value)
        elif value is None:
            raise SystemdPyError(f'{self.__class__.__name__} value cannot be None')
        elif isinstance(value, dict):
            raise SystemdPyError(f'{self.__class__.__name__} does not support dict')
        if isinstance(value, Enum):
            return str(value.value)
        else:
            return str(value)

    def __str__(self):
        if all(getattr(self, s) is None for s in self.__fields__):
            raise SystemdPyError(f'{self.__class__.__name__} is empty')

        text = f'[{self.__class__.__name__}]\n'
        dict_ = self.dict(by_alias=True, exclude_none=True)

        extra = dict_.pop('Extra', None)
        for k, v in dict_.items():
            dict_[k] = self._to_str(v)

        text += "\n".join([f'{k}={v!r}' for k, v in dict_.items()])

        if extra:
            for k, v in extra.items():
                text += f'\n{k}={v}'

        return text

    def __repr__(self):
        return self.__str__()

    def __add__(self, other):
        if isinstance(other, Section):
            return self.__str__() + "\n\n" + other.__str__()
        return TypeError(f'unsupported operand type(s) for +: {self.__class__.__name__} and {other.__class__.__name__}')

    def __radd__(self, other):
        return self.__add__(other)

    class Config:
        allow_population_by_field_name = True

        @classmethod
        def alias_generator(cls, string: str) -> str:
            """
            Convert to CamelCase

            Args:
                string (str, required): String to convert

            Returns:
                str: CamelCase string
            """

            return ''.join(word.capitalize() for word in string.split('_'))



from typing import Optional
from typing import List
from typing import Union
from pydantic import Field

from ._models import Section


class Install(Section):
    """
    Systemd [Install] Section Directives
    """

    wanted_by: Optional[Union[str, List[str]]] = Field(
        None,
        title='WantedBy',
        description='A space-separated list of units that should be started when this unit is.',
    )
    required_by: Optional[Union[str, List[str]]] = Field(
        None,
        title='RequiredBy',
        description='A space-separated list of units that must be started when this unit is.'
    )
    alias: Optional[Union[str, List[str]]] = Field(
        None,
        title='Alias',
        description='A space-separated list of additional names for this unit.'
    )
    also: Optional[Union[str, List[str]]] = Field(
        None,
        title='Also',
        description='A space-separated list of units that should be installed when this unit is.'
    )
    default_instance: Optional[str] = Field(
        None,
        title='DefaultInstance',
        description='The default instance name for this template unit.'
    )

    class Config:
        fields = {
            'WantedBy': 'wanted_by',
            'RequiredBy': 'required_by',
            'Alias': 'alias',
            'Also': 'also',
            'DefaultInstance': 'default_instance'
        }



from enum import Enum as _Enum


class Enum(_Enum):
    def __str__(self):
        return self.value

    def __repr__(self):
        return self.value

    def __eq__(self, other):
        if isinstance(other, str):
            return self.value == other
        if isinstance(other, Enum):
            return self.value == other.value

        raise TypeError(f"Cannot compare {self.__class__.__name__} with {other.__class__.__name__}")



"""
# Install Builder 🏗️
Builder for the `install` section of a systemd service file.
"""

from typing import Optional
from typing import List

from ._builder import _Builder
from ..core.models import Install


class InstallBuilder(_Builder):
    """
    Builder for the `install` section of a systemd service file.
    """

    __slots__ = (
        '_wanted_by',
        '_required_by',
        '_alias',
        '_also',
        '_default_instance',
    )

    @property
    def allowed_none_fields(self) -> List[str]:
        """
        Returns a list of fields that are allowed to be None


        :return: List of fields that are allowed to be None
        :rtype: List[str]
        """

        return [
            '_wanted_by', '_required_by', '_alias', '_also', '_default_instance'
        ]

    def build(self) -> Install:
        """
        Builds the `install` section of a systemd service file.

        :return: The `install` section of a systemd service file
        :rtype: Install
        """

        self._check()
        return Install(
            wanted_by=self._wanted_by,
            required_by=self._required_by,
            alias=self._alias,
            also=self._also,
            default_instance=self._default_instance
        )

    def __init__(self):
        self._wanted_by: Optional[List[str]] = None
        self._required_by: Optional[List[str]] = None
        self._alias: Optional[List[str]] = None
        self._also: Optional[List[str]] = None
        self._default_instance: Optional[str] = None

    def with_wanted_by(self, wanted_by: List[str]) -> 'InstallBuilder':
        """
        Sets the `wanted_by` field of the `install` section of a systemd service file.

        :param wanted_by: The `wanted_by` field of the `install` section of a systemd service file
        :type wanted_by: List[str]

        :return: The `InstallBuilder` object
        :rtype: InstallBuilder
        """

        self._wanted_by = wanted_by
        return self

    def with_required_by(self, required_by: List[str]) -> 'InstallBuilder':
        """
        Sets the `required_by` field of the `install` section of a systemd service file.

        :param required_by: The `required_by` field of the `install` section of a systemd service file
        :type required_by: List[str]

        :return: The `InstallBuilder` object
        :rtype: InstallBuilder
        """

        self._required_by = required_by
        return self

    def with_alias(self, alias: List[str]) -> 'InstallBuilder':
        """
        Sets the `alias` field of the `install` section of a systemd service file.

        :param alias: The `alias` field of the `install` section of a systemd service file
        :type alias: List[str]

        :return: The `InstallBuilder` object
        :rtype: InstallBuilder
        """

        self._alias = alias
        return self

    def with_also(self, also: List[str]) -> 'InstallBuilder':
        """
        Sets the `also` field of the `install` section of a systemd service file.

        :param also: The `also` field of the `install` section of a systemd service file
        :type also: List[str]

        :return: The `InstallBuilder` object
        :rtype: InstallBuilder
        """

        self._also = also
        return self

    def with_default_instance(self, default_instance: str) -> 'InstallBuilder':
        """
        Sets the `default_instance` field of the `install` section of a systemd service file.

        :param default_instance: The `default_instance` field of the `install` section of a systemd service file
        :type default_instance: str

        :return: The `InstallBuilder` object
        :rtype: InstallBuilder
        """

        self._default_instance = default_instance
        return self



from typing import Union
from pathlib import Path


def get_file(name: str, path: Union[str, Path]) -> Path:
    """
    Validate path
    """

    if not name.endswith('.service'):
        name += '.service'

    if isinstance(path, str):
        path = Path(path)

    path = path.resolve()

    if path.is_file():
        path = path.parent

    return path / name



from typing import Union
from typing import Literal

from ..enums import Type
from ..enums import Restart


TypeType = Union[
    Type,
    Literal["simple"],
    Literal["forking"],
    Literal["oneshot"],
    Literal["dbus"],
    Literal["notify"],
    Literal["idle"]
]

RestartType = Union[
    Restart,
    Literal["no"],
    Literal["on_failure", "on-failure"],
    Literal["on_abnormal", "on-abnormal"],
    Literal["on_watchdog", "on-watchdog"],
    Literal["on_abort", "on-abort"],
]



"""
# Builder Base Class
"""

from typing import List
from abc import abstractmethod, ABC


class _Builder(ABC):
    @abstractmethod
    def build(self):
        """
        Builds the object
        """

        raise NotImplementedError

    def __call__(self):
        """
        Builds the object
        """

        self._check()
        return self.build()

    @property
    @abstractmethod
    def allowed_none_fields(self) -> List[str]:
        """
        Returns a list of fields that are allowed to be None
        """

        raise NotImplementedError

    def _make_dict(self) -> dict:
        """
        Returns a dict of the object
        """

        return {s: getattr(self, s) for s in self.__slots__}

    def _check(self):
        """
        if value is None and field is not allowed to be None, raise TypeError
        """

        for s in self.__slots__:
            if getattr(self, s) is ...:
                setattr(self, s, None)
            if getattr(self, s) in (None, '', [], (), {}, ..., Ellipsis) and s not in self.allowed_none_fields:
                raise TypeError(f'{s[1:]} is not allowed to be None')

    def __repr__(self):
        return f'{self.__class__.__name__}({self._make_dict()})'

    def __str__(self):
        return f'{self.__class__.__name__}({self._make_dict()})'



"""
# Socket Builder 🏗️
Builder for the `socket` section of a systemd service file.
"""

from typing import Optional
from typing import List

from ._builder import _Builder
from ..core.models import Socket


class SocketBuilder(_Builder):
    """
    Builder for the `socket` section of a systemd service file.
    """

    __slots__ = (
        "_listen_stream",
        "_listen_datagram",
        "_listen_sequential_packet",
        "_listen_fifo",
        "_accept",
        "_socket_user",
        "_socket_group",
        "_socket_mode",
        "_service",
    )

    @property
    def allowed_none_fields(self) -> List[str]:
        """
        Returns a list of fields that are allowed to be None


        :return: List of fields that are allowed to be None
        :rtype: List[str]
        """

        return [
            "_listen_stream", "_listen_datagram", "_listen_sequential_packet",
            "_listen_fifo", "_accept", "_socket_user", "_socket_group",
            "_socket_mode", "_service",
        ]

    def build(self) -> Socket:
        """
        Builds the `socket` section of a systemd service file.

        :return: The `socket` section of a systemd service file
        :rtype: Socket
        """

        self._check()
        return Socket(
            listen_stream=self._listen_stream,
            listen_datagram=self._listen_datagram,
            listen_sequential_packet=self._listen_sequential_packet,
            listen_fifo=self._listen_fifo,
            accept=self._accept,
            socket_user=self._socket_user,
            socket_group=self._socket_group,
            socket_mode=self._socket_mode,
            service=self._service,
        )

    def __init__(self):
        self._listen_stream: Optional[List[str]] = None
        self._listen_datagram: Optional[List[str]] = None
        self._listen_sequential_packet: Optional[List[str]] = None
        self._listen_fifo: Optional[List[str]] = None
        self._accept: Optional[bool] = None
        self._socket_user: Optional[str] = None
        self._socket_group: Optional[str] = None
        self._socket_mode: Optional[str] = None
        self._service: Optional[str] = None

    def with_listen_stream(self, listen_stream: List[str]) -> 'SocketBuilder':
        """
        Sets the `listen_stream` field of the `socket` section of a systemd service file.

        :param listen_stream: The `listen_stream` field of the `socket` section of a systemd service file
        :type listen_stream: List[str]

        :return: The SocketBuilder object
        :rtype: SocketBuilder
        """

        self._listen_stream = listen_stream
        return self

    def with_listen_datagram(self, listen_datagram: List[str]) -> 'SocketBuilder':
        """
        Sets the `listen_datagram` field of the `socket` section of a systemd service file.

        :param listen_datagram: The `listen_datagram` field of the `socket` section of a systemd service file
        :type listen_datagram: List[str]

        :return: The SocketBuilder object
        :rtype: SocketBuilder
        """

        self._listen_datagram = listen_datagram
        return self

    def with_listen_sequential_packet(self, listen_sequential_packet: List[str]) -> 'SocketBuilder':
        """
        Sets the `listen_sequential_packet` field of the `socket` section of a systemd service file.

        :param listen_sequential_packet: The `listen_sequential_packet` field of the `socket` section of a systemd service file
        :type listen_sequential_packet: List[str]

        :return: The SocketBuilder object
        :rtype: SocketBuilder
        """

        self._listen_sequential_packet = listen_sequential_packet
        return self

    def with_listen_fifo(self, listen_fifo: List[str]) -> 'SocketBuilder':
        """
        Sets the `listen_fifo` field of the `socket` section of a systemd service file.

        :param listen_fifo: The `listen_fifo` field of the `socket` section of a systemd service file
        :type listen_fifo: List[str]

        :return: The SocketBuilder object
        :rtype: SocketBuilder
        """

        self._listen_fifo = listen_fifo
        return self

    def with_accept(self, accept: bool) -> 'SocketBuilder':
        """
        Sets the `accept` field of the `socket` section of a systemd service file.

        :param accept: The `accept` field of the `socket` section of a systemd service file
        :type accept: bool

        :return: The SocketBuilder object
        :rtype: SocketBuilder
        """

        self._accept = accept
        return self

    def with_socket_user(self, socket_user: str) -> 'SocketBuilder':
        """
        Sets the `socket_user` field of the `socket` section of a systemd service file.

        :param socket_user: The `socket_user` field of the `socket` section of a systemd service file
        :type socket_user: str

        :return: The SocketBuilder object
        :rtype: SocketBuilder
        """

        self._socket_user = socket_user
        return self

    def with_socket_group(self, socket_group: str) -> 'SocketBuilder':
        """
        Sets the `socket_group` field of the `socket` section of a systemd service file.

        :param socket_group: The `socket_group` field of the `socket` section of a systemd service file
        :type socket_group: str

        :return: The SocketBuilder object
        :rtype: SocketBuilder
        """

        self._socket_group = socket_group
        return self

    def with_socket_mode(self, socket_mode: str) -> 'SocketBuilder':
        """
        Sets the `socket_mode` field of the `socket` section of a systemd service file.

        :param socket_mode: The `socket_mode` field of the `socket` section of a systemd service file
        :type socket_mode: str

        :return: The SocketBuilder object
        :rtype: SocketBuilder
        """

        self._socket_mode = socket_mode
        return self

    def with_service(self, service: str) -> 'SocketBuilder':
        """
        Sets the `service` field of the `socket` section of a systemd service file.

        :param service: The `service` field of the `socket` section of a systemd service file
        :type service: str

        :return: The SocketBuilder object
        :rtype: SocketBuilder
        """

        self._service = service
        return self



from typing import Optional
from typing import List
from typing import Union
from pydantic import Field

from ._models import Section
from ..types import TypeType, RestartType


class Service(Section):
    """
    Systemd [Service] Section Directives
    """

    type: Optional[TypeType] = Field(
        None,
        title="Type",
        description="The type of service to start. One of simple, forking, oneshot, dbus, notify, idle, or "
                    "exec. If this option is not specified, simple is assumed."
    )
    remain_after_exit: Optional[bool] = Field(
        None,
        title="RemainAfterExit",
        description="This directive is commonly used with the oneshot type. It indicates that "
                    "the service should be considered active even after the process exits."
    )
    pid_file: Optional[str] = Field(
        None,
        title="PIDFile",
        description="If the service type is marked as “forking”, this directive is used to set the "
                    "path of the file that should contain the process ID number of the main child "
                    "that should be monitored."
    )
    bus_name: Optional[str] = Field(
        None,
        title="BusName",
        description="This directive should be set to the D-Bus bus name that the service will attempt "
                    "to acquire when using the “dbus” service type."
    )
    notify_access: Optional[str] = Field(
        None,
        title="NotifyAccess",
        description='This specifies access to the socket that should be used to listen for notifications '
                    'when the “notify” service type is selected This can be “none”, “main”, or "all. '
                    'The default, “none”, ignores all status messages. The “main” option will listen to '
                    'messages from the main process and the “all” option will cause all members of '
                    'the service’s control group to be processed.'
    )
    exec_start: Optional[Union[str, List[str]]] = Field(
        None,
        title="ExecStart",
        description="This specifies the full path and the arguments of the command to be executed to "
                    "start the process. This may only be specified once (except for “oneshot” services). "
                    "If the path to the command is preceded by a dash “-” character, non-zero exit "
                    "statuses will be accepted without marking the unit activation as failed."
    )
    exec_start_pre: Optional[Union[str, List[str]]] = Field(
        None,
        title="ExecStartPre",
        description="This can be used to provide additional commands that should be executed before "
                    "the main process is started. This can be used multiple times. Again, "
                    "commands must specify a full path and they can be preceded by “-” to indicate that "
                    "the failure of the command will be tolerated."
    )
    exec_start_post: Optional[Union[str, List[str]]] = Field(
        None,
        title="ExecStartPost",
        description="This has the same exact qualities as `ExecStartPre=` except that it specifies "
                    "commands that will be run after the main process is started."
    )
    exec_reload: Optional[Union[str, List[str]]] = Field(
        None,
        title="ExecReload",
        description="This optional directive indicates the command necessary to reload the "
                    "configuration of the service if available"
    )
    exec_stop: Optional[Union[str, List[str]]] = Field(
        None,
        title="ExecStop",
        description="This indicates the command needed to stop the service. If this is not given, "
                    "the process will be killed immediately when the service is stopped."
    )
    exec_stop_post: Optional[Union[str, List[str]]] = Field(
        None,
        title="ExecStopPost",
        description="This can be used to specify commands to execute following the stop command"
    )
    restart_sec: Optional[int] = Field(
        None,
        title='RestartSec',
        description='If automatically restarting the service is enabled, this specifies the amount of time to '
                    'wait before attempting to restart the service.'
    )
    restart: Optional[RestartType] = Field(
        None,
        title='Restart',
        description='This indicates the circumstances under which systemd will attempt to automatically restart the '
                    'service. This can be set to values like “always”, “on-success”, “on-failure”, “on-abnormal”, '
                    '“on-abort”, or “on-watchdog”. These will trigger a restart according to the way that the service '
                    'was stopped.'
    )
    timeout_sec: Optional[int] = Field(
        None,
        title='TimeoutSec',
        description='This configures the amount of time that systemd will wait when stopping or stopping the service '
                    'before marking it as failed or forcefully killing it. You can set separate timeouts with '
                    'TimeoutStartSec= and TimeoutStopSec= as well.'
    )
    user: Optional[str] = Field(
        None,
        title='User',
        description='This specifies the user that the service should be run as. This can be a user name or a numeric '
                    'UID. If this is not specified, the service will be run as root.'
    )
    group: Optional[str] = Field(
        None,
        title='Group',
        description='This specifies the group that the service should be run as. This can be a group name or a numeric '
                    'GID. If this is not specified, the service will be run as root.'
    )
    nice: Optional[int] = Field(
        None,
        title='Nice',
        description='This specifies the nice level of the service. This can be a value between -20 and 19. '
                    'The default is 0.'
    )

    class Config:
        fields = {
            'remain_after_exit': 'RemainAfterExit',
            'pid_file': 'PIDFile',
            'bus_name': 'BusName',
            'notify_access': 'NotifyAccess',
            'exec_start': 'ExecStart',
            'exec_start_pre': 'ExecStartPre',
            'exec_start_post': 'ExecStartPost',
            'exec_reload': 'ExecReload',
            'exec_stop': 'ExecStop',
            'exec_stop_post': 'ExecStopPost',
            'restart_sec': 'RestartSec',
            'timeout_sec': 'TimeoutSec',
            'user': 'User',
            'group': 'Group',
            'nice': 'Nice',
        }



from typing import Optional
from typing import List
from typing import Union
from pydantic import Field

from ._models import Section


class Unit(Section):
    """
    Systemd [Unit] Section Directives
    """

    description: str = Field(
        None,
        title='Description',
        description='A short, one-line description of the service.'
    )
    documentation: Optional[str] = Field(
        None,
        title='Documentation',
        description='A space-separated list of URLs to documentation for this unit.'
    )
    requires: Optional[Union[str, List[str]]] = Field(
        None,
        title='Requires',
        description='A space-separated list of units that must be started before this unit.'
    )
    wants: Optional[Union[str, List[str]]] = Field(
        None,
        title='Wants',
        description='A space-separated list of units that should be started if this unit is.'
    )
    binds_to: Optional[Union[str, List[str]]] = Field(
        None,
        title='BindsTo',
        description='A space-separated list of units that this unit is bound to.'
    )
    before: Optional[Union[str, List[str]]] = Field(
        None,
        title='Before',
        description='A space-separated list of units that should be stopped if this unit is.'
    )
    after: Optional[Union[str, List[str]]] = Field(
        None,
        title='After',
        description='A space-separated list of units that must be stopped before this unit is.'
    )
    conflicts: Optional[Union[str, List[str]]] = Field(
        None,
        title='Conflicts',
        description='A space-separated list of units that this unit conflicts with.'
    )
    condition: Optional[str] = Field(
        None,
        title='Condition',
        description='A boolean expression that must be true for the unit to be started.'
    )
    assert_: Optional[str] = Field(
        None,
        title='Assert',
        description='A boolean expression that must be true for the unit to be started.'
    )

    class Config:
        fields = {
            'Description': 'description',
            'Documentation': 'documentation',
            'Requires': 'requires',
            'Wants': 'wants',
            'BindsTo': 'binds_to',
            'Before': 'before',
            'After': 'after',
            'Conflicts': 'conflicts',
            'Condition': 'condition',
            'Assert': 'assert_'
        }



from typing import Optional
from typing import List
from typing import Union
from pydantic import Field

from ._models import Section


class Socket(Section):
    """
    Systemd [Socket] Section Directives
    """

    listen_stream: Optional[Union[str, List[str]]] = Field(
        None,
        title="ListenStream",
        description="This defines an address for a stream socket which supports sequential, reliable "
                    "communication. Services that use TCP should use this socket type."
    )
    listen_datagram: Optional[Union[str, List[str]]] = Field(
        None,
        title="ListenDatagram",
        description="This defines an address for a datagram socket which supports fast, unreliable "
                    "communication packets. Services that use UDP should set this socket type."
    )
    listen_sequential_packet: Optional[Union[str, List[str]]] = Field(
        None,
        title="ListenSequentialPacket",
        description="This defines an address for sequential, reliable communication with max length "
                    "datagrams that preserves message boundaries. This is found most often for Unix sockets."
    )
    listen_fifo: Optional[Union[str, List[str]]] = Field(
        None,
        title="ListenFIFO",
        description="Along with the other listening types, you can also specify a FIFO buffer instead of a socket."
    )
    accept: Optional[bool] = Field(
        None,
        title="Accept",
        description="This determines whether an additional instance of the service will be each connection. "
                    "If set to false (the default), one instance will handle all connections."
    )
    socket_user: Optional[str] = Field(
        None,
        title="SocketUser",
        description="With a Unix socket, specifies the owner of the socket. This will be the root user if left unset."
    )
    socket_group: Optional[str] = Field(
        None,
        title="SocketGroup",
        description="With a Unix socket, specifies the group owner of the socket. This will be the root "
                    "group if neither this or the above are set. If only the SocketUser= is set, "
                    "systemd will try to find a matching group."
    )
    socket_mode: Optional[str] = Field(
        None,
        title="SocketMode",
        description="For Unix sockets or FIFO buffers, this sets the permissions on the created entity."
    )
    service: Optional[str] = Field(
        None,
        title="Service",
        description="If the service name does not match the .socket name, the service can be "
                    "specified with this directive."
    )

    class Config:
        fields = {
            'ListenStream': 'listen_stream',
            'ListenDatagram': 'listen_datagram',
            'ListenSequentialPacket': 'listen_sequential_packet',
            'ListenFIFO': 'listen_fifo',
            'Accept': 'accept',
            'SocketUser': 'socket_user',
            'SocketGroup': 'socket_group',
            'SocketMode': 'socket_mode',
            'Service': 'service'
        }



from typing import Literal

from ._enums import _Enum


class Restart(_Enum):
    """
    Service Restart
    """

    no: Literal['no'] = 'no'
    on_failure: Literal['on-failure'] = 'on-failure'
    on_abnormal: Literal['on-abnormal'] = 'on-abnormal'
    on_watchdog: Literal['on-watchdog'] = 'on-watchdog'
    on_abort: Literal['on-abort'] = 'on-abort'



"""
# Service Builder 🏗️
Builder for the `service` section of a systemd service file.

"""

from typing import Optional
from typing import List

from ._builder import _Builder
from ..core.models import Service
from ..core.types import TypeType, RestartType


class ServiceBuilder(_Builder):
    """
    Builder for the `service` section of a systemd service file.
    """

    __slots__ = (
        '_type',
        '_remain_after_exit',
        '_pid_file',
        '_bus_name',
        '_notify_access',
        '_exec_start',
        '_exec_start_pre',
        '_exec_start_post',
        '_exec_reload',
        '_exec_stop',
        '_exec_stop_post',
        '_restart_sec',
        '_restart',
        '_timeout_sec'
    )

    @property
    def allowed_none_fields(self) -> List[str]:
        """
        Returns a list of fields that are allowed to be None


        :return: List of fields that are allowed to be None
        :rtype: List[str]
        """

        return [
            "_type", "_remain_after_exit", "_pid_file", "_bus_name", "_notify_access",
            "_exec_start", "_exec_start_pre", "_exec_start_post", "_exec_reload", "_exec_stop",
            "_exec_stop_post", "_restart_sec", "_restart", "_timeout_sec",
        ]

    def __init__(self):
        self._type: Optional[TypeType] = None
        self._remain_after_exit: Optional[bool] = None
        self._pid_file: Optional[str] = None
        self._bus_name: Optional[str] = None
        self._notify_access: Optional[str] = None
        self._exec_start: Optional[List[str]] = None
        self._exec_start_pre: Optional[List[str]] = None
        self._exec_start_post: Optional[List[str]] = None
        self._exec_reload: Optional[List[str]] = None
        self._exec_stop: Optional[List[str]] = None
        self._exec_stop_post: Optional[List[str]] = None
        self._restart_sec: Optional[int] = None
        self._restart: Optional[RestartType] = None
        self._timeout_sec: Optional[int] = None

    def build(self) -> Service:
        """
        Builds the `service` section of a systemd service file.

        :return: The `service` section of a systemd service file
        :rtype: Service
        """

        self._check()
        return Service(
            type=self._type,
            remain_after_exit=self._remain_after_exit,
            pid_file=self._pid_file,
            bus_name=self._bus_name,
            notify_access=self._notify_access,
            exec_start=self._exec_start,
            exec_start_pre=self._exec_start_pre,
            exec_start_post=self._exec_start_post,
            exec_reload=self._exec_reload,
            exec_stop=self._exec_stop,
            exec_stop_post=self._exec_stop_post,
            restart_sec=self._restart_sec,
            restart=self._restart,
            timeout_sec=self._timeout_sec
        )

    def with_type(self, type: TypeType) -> 'ServiceBuilder':
        """
        Sets the `type` field of the `service` section of a systemd service file.

        :param type: The `type` field of the `service` section of a systemd service file
        :type type: TypeType

        :return: The `ServiceBuilder` object
        :rtype: ServiceBuilder
        """

        self._type = type
        return self

    def with_remain_after_exit(self, remain_after_exit: bool) -> 'ServiceBuilder':
        """
        Sets the `remain_after_exit` field of the `service` section of a systemd service file.

        :param remain_after_exit: The `remain_after_exit` field of the `service` section of a systemd service file
        :type remain_after_exit: bool

        :return: The `ServiceBuilder` object
        :rtype: ServiceBuilder
        """

        self._remain_after_exit = remain_after_exit
        return self

    def with_pid_file(self, pid_file: str) -> 'ServiceBuilder':
        """
        Sets the `pid_file` field of the `service` section of a systemd service file.

        :param pid_file: The `pid_file` field of the `service` section of a systemd service file
        :type pid_file: str

        :return: The `ServiceBuilder` object
        :rtype: ServiceBuilder
        """

        self._pid_file = pid_file
        return self

    def with_bus_name(self, bus_name: str) -> 'ServiceBuilder':
        """
        Sets the `bus_name` field of the `service` section of a systemd service file.

        :param bus_name: The `bus_name` field of the `service` section of a systemd service file
        :type bus_name: str

        :return: The `ServiceBuilder` object
        :rtype: ServiceBuilder
        """

        self._bus_name = bus_name
        return self

    def with_notify_access(self, notify_access: str) -> 'ServiceBuilder':
        """
        Sets the `notify_access` field of the `service` section of a systemd service file.

        :param notify_access: The `notify_access` field of the `service` section of a systemd service file
        :type notify_access: str

        :return: The `ServiceBuilder` object
        :rtype: ServiceBuilder
        """

        self._notify_access = notify_access
        return self

    def with_exec_start(self, exec_start: List[str]) -> 'ServiceBuilder':
        """
        Sets the `exec_start` field of the `service` section of a systemd service file.

        :param exec_start: The `exec_start` field of the `service` section of a systemd service file
        :type exec_start: List[str]

        :return: The `ServiceBuilder` object
        :rtype: ServiceBuilder
        """

        self._exec_start = exec_start
        return self

    def with_exec_start_pre(self, exec_start_pre: List[str]) -> 'ServiceBuilder':
        """
        Sets the `exec_start_pre` field of the `service` section of a systemd service file.

        :param exec_start_pre: The `exec_start_pre` field of the `service` section of a systemd service file
        :type exec_start_pre: List[str]

        :return: The `ServiceBuilder` object
        :rtype: ServiceBuilder
        """

        self._exec_start_pre = exec_start_pre
        return self

    def with_exec_start_post(self, exec_start_post: List[str]) -> 'ServiceBuilder':
        """
        Sets the `exec_start_post` field of the `service` section of a systemd service file.

        :param exec_start_post: The `exec_start_post` field of the `service` section of a systemd service file
        :type exec_start_post: List[str]

        :return: The `ServiceBuilder` object
        :rtype: ServiceBuilder
        """

        self._exec_start_post = exec_start_post
        return self

    def with_exec_reload(self, exec_reload: List[str]) -> 'ServiceBuilder':
        """
        Sets the `exec_reload` field of the `service` section of a systemd service file.

        :param exec_reload: The `exec_reload` field of the `service` section of a systemd service file
        :type exec_reload: List[str]

        :return: The `ServiceBuilder` object
        :rtype: ServiceBuilder
        """

        self._exec_reload = exec_reload
        return self

    def with_exec_stop(self, exec_stop: List[str]) -> 'ServiceBuilder':
        """
        Sets the `exec_stop` field of the `service` section of a systemd service file.

        :param exec_stop: The `exec_stop` field of the `service` section of a systemd service file
        :type exec_stop: List[str]

        :return: The `ServiceBuilder` object
        :rtype: ServiceBuilder
        """

        self._exec_stop = exec_stop
        return self

    def with_exec_stop_post(self, exec_stop_post: List[str]) -> 'ServiceBuilder':
        """
        Sets the `exec_stop_post` field of the `service` section of a systemd service file.

        :param exec_stop_post: The `exec_stop_post` field of the `service` section of a systemd service file
        :type exec_stop_post: List[str]

        :return: The `ServiceBuilder` object
        :rtype: ServiceBuilder
        """

        self._exec_stop_post = exec_stop_post
        return self

    def with_restart_sec(self, restart_sec: int) -> 'ServiceBuilder':
        """
        Sets the `restart_sec` field of the `service` section of a systemd service file.

        :param restart_sec: The `restart_sec` field of the `service` section of a systemd service file
        :type restart_sec: int

        :return: The `ServiceBuilder` object
        :rtype: ServiceBuilder
        """

        self._restart_sec = restart_sec
        return self

    def with_restart(self, restart: RestartType) -> 'ServiceBuilder':
        """
        Sets the `restart` field of the `service` section of a systemd service file.

        :param restart: The `restart` field of the `service` section of a systemd service file
        :type restart: RestartType

        :return: The `ServiceBuilder` object
        :rtype: ServiceBuilder
        """

        self._restart = restart
        return self

    def with_timeout_sec(self, timeout_sec: int) -> 'ServiceBuilder':
        """
        Sets the `timeout_sec` field of the `service` section of a systemd service file.

        :param timeout_sec: The `timeout_sec` field of the `service` section of a systemd service file
        :type timeout_sec: int

        :return: The `ServiceBuilder` object
        :rtype: ServiceBuilder
        """

        self._timeout_sec = timeout_sec
        return self



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['systemd_py',
 'systemd_py.builders',
 'systemd_py.core',
 'systemd_py.core.enums',
 'systemd_py.core.models',
 'systemd_py.core.types',
 'systemd_py.utils']

package_data = \
{'': ['*']}

install_requires = \
['mkdocs-gen-files>=0.4.0,<0.5.0',
 'mkdocs-git-committers-plugin-2>=1.1.1,<2.0.0',
 'mkdocs-git-revision-date-localized-plugin>=1.1.0,<2.0.0',
 'mkdocs-literate-nav>=0.5.0,<0.6.0',
 'mkdocs-material>=8.5.11,<9.0.0',
 'mkdocs-section-index>=0.3.4,<0.4.0',
 'mkdocs>=1.4.2,<2.0.0',
 'mkdocstrings[python]>=0.19.0,<0.20.0',
 'pydantic>=1.10.2,<2.0.0']

setup_kwargs = {
    'name': 'systemd-py',
    'version': '0.1.0',
    'description': '',
    'long_description': '',
    'author': 'amiwrpremium',
    'author_email': 'amiwrpremium@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



"""
# Builders for systemd_py

Set of builder tools for help ypu build each
section for systemd service files.

&nbsp;

[Install Builder](install) - Build the `install` section for a service file.

[Service Builder](service) - Build the `service` section for a service file.

[Socket Builder](socket) - Build the `socket` section for a service file.

[Unit Builder](unit) - Build the `unit` section for a service file.
"""

from .install import InstallBuilder
from .service import ServiceBuilder
from .socket import SocketBuilder
from .unit import UnitBuilder



import subprocess


class Daemon:
    """
    Systemd daemon
    """

    def __init__(self, name: str):
        """
        Systemd daemon

        Args:
            name (str, required): daemon name
        """

        self._name = name

    @staticmethod
    def reload() -> None:
        """
        Reload systemd daemon
        """

        subprocess.run(['systemctl', 'daemon-reload'], check=True)
        print('Reloaded systemd daemon')

    def enable(self) -> None:
        """
        Enable systemd service
        """

        subprocess.run(['systemctl', 'enable', self._name], check=True)
        print(f'Enabled {self._name}')

    def disable(self) -> None:
        """
        Disable systemd service
        """

        subprocess.run(['systemctl', 'disable', self._name], check=True)
        print(f'Disabled {self._name}')

    def start(self) -> None:
        """
        Start systemd service
        """

        subprocess.run(['systemctl', 'start', self._name], check=True)
        print(f'Started {self._name}')

    def stop(self) -> None:
        """
        Stop systemd service
        """

        subprocess.run(['systemctl', 'stop', self._name], check=True)
        print(f'Stopped {self._name}')

    def restart(self) -> None:
        """
        Restart systemd service
        """

        subprocess.run(['systemctl', 'restart', self._name], check=True)
        print(f'Restarted {self._name}')

    def status(self) -> None:
        """
        Status systemd service
        """

        subprocess.run(['systemctl', 'status', self._name], check=True)
        print(f'Status {self._name}')

    def logs(self) -> None:
        """
        Logs systemd service
        """

        subprocess.run(['journalctl', '-u', self._name], check=True)
        print(f'Logs {self._name}')

    def __str__(self) -> str:
        return self._name

    def __repr__(self) -> str:
        return self.__str__()
